==> 0.0.15.tar.gz: lib/src/widgets/tables/table_frame.dart <==
import 'dart:convert';
import 'dart:typed_data';
import "package:collection/collection.dart";
import 'package:verttice_ui/src/const/box_sizes.dart';
import 'package:verttice_ui/src/const/icons.dart';
import 'package:verttice_ui/src/utils/download.dart';
import 'package:verttice_ui/src/widgets/fade_edges.dart';
import 'package:verttice_ui/src/widgets/form_factory/form_factory.dart';
import 'package:verttice_ui/src/widgets/tables/table_content.dart';
import 'package:verttice_ui/src/widgets/tables/table_utils.dart';
import "package:csv/csv.dart";
import "package:dropdown_search/dropdown_search.dart";
import "package:flutter/material.dart";
import "package:flutter/scheduler.dart";
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/src/widgets/themed_divider.dart';

class TableFrame extends StatefulWidget {
  final bool showCopyRowButton;
  final void Function(List<Map<String, dynamic>>)? selectedListCallback;
  final List<FormItem> columns;
  final List<Map<String, dynamic>> data;
  final double frameWidth;
  final double frameHeight;
  final List<int> rowsPerPageOptions;
  final void Function(
    Map<String, dynamic>? rowData,
    int rowIndex,
  ) handleSelectedRow;
  final List<Widget>? tableActions;
  final double? tableActionPrecentageWidth;
  final String title;

  const TableFrame({
    Key? key,
    required this.showCopyRowButton,
    required this.selectedListCallback,
    required this.columns,
    required this.data,
    required this.frameWidth,
    required this.frameHeight,
    required this.rowsPerPageOptions,
    required this.handleSelectedRow,
    required this.title,
    this.tableActions,
    this.tableActionPrecentageWidth,
  }) : super(key: key);

  @override
  _TableFrameState createState() => _TableFrameState();
}

class _TableFrameState extends State<TableFrame> {
  final _debouncer = Debouncer(milliseconds: 500);
  final double _topBarHeight = 40;
  final double _bottomBarHeight = 40;
  final double _seperator = 2;
  late int _pageLength;
  int _currentPage = 0;
  late List<Map<String, dynamic>> _data;
  late List<Map<String, dynamic>> _filterdData;
  String _search = "";
  Map<String, List<FilterField>> _filters = {};

  @override
  void initState() {
    _pageLength = widget.rowsPerPageOptions[0];
    _data = widget.data;
    _filterdData = _data;
    super.initState();
  }

  @override
  void didUpdateWidget(covariant TableFrame oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const DeepCollectionEquality().equals(_data, widget.data)) {
      _data = widget.data;
      _filterdData = _filterResult(_search, _filters, _data);
    }
  }

  @override
  void setState(VoidCallback fn) {
    if (mounted) {
      SchedulerBinding.instance?.addPostFrameCallback((_) {
        super.setState(fn);
      });
    }
  }

  List<Map<String, dynamic>> _filterResult(
    String search,
    Map<String, List<FilterField>> filters,
    List<Map<String, dynamic>> data,
  ) {
    final result = _searchResult(search, _data).where((element) {
      bool include = true;
      for (var entry in _filters.entries) {
        for (var filter in entry.value) {
          final applFil = applayTableFilter(
            filter: filter.value,
            cellData: element[entry.key],
          );
          if (!applFil) {
            include = false;
          }
        }
      }
      return include;
    }).toList();

    return result;
  }

  List<Map<String, dynamic>> _searchResult(
    String search,
    List<Map<String, dynamic>> data,
  ) {
    return search == ""
        ? data
        : data
            .where(
              (element) => json.encode(element).toLowerCase().contains(
                    search.toLowerCase(),
                  ),
            )
            .toList();
  }

  void _dataToCsv() async {
    List<String> header = [];

    for (var element in widget.columns) {
      if (element.field.displayName != "") {
        header.add(element.field.displayName);
      }
    }

    final List<List<String>> data = [];

    data.add(header);

    _filterdData.asMap().forEach((rowIndex, rowData) {
      final List<String> currenRow = [];

      widget.columns.asMap().forEach((index, column) {
        currenRow.add(visualDataProcessor(column, rowData).value);
      });

      data.add(currenRow);
    });

    String csv = const ListToCsvConverter().convert(data);
    await Download.memory(
      data: Uint8List.fromList(csv.codeUnits),
      fileType: VFileType.csv,
    );
  }

  @override
  Widget build(BuildContext context) {
    final dataLength = _filterdData.length;
    final maxPages = (dataLength ~/ _pageLength);
    final itemsLength = dataLength;
    double titleWidth = widget.frameWidth / 3;
    double userActionsWidget = widget.frameWidth / 3;
    double coreActionsWidth = widget.frameWidth / 3;
    if (widget.tableActionPrecentageWidth != null &&
        widget.tableActionPrecentageWidth! <= 0.7) {
      userActionsWidget =
          widget.frameWidth * widget.tableActionPrecentageWidth!;

      titleWidth =
          widget.frameWidth * ((1.0 - widget.tableActionPrecentageWidth!) / 2);
      coreActionsWidth =
          widget.frameWidth * ((1.0 - widget.tableActionPrecentageWidth!) / 2);
    }

    return SizedBox(
      width: widget.frameWidth,
      height: widget.frameHeight,
      child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisAlignment: MainAxisAlignment.start,
          children: [
            SizedBox(
              width: widget.frameWidth,
              height: _topBarHeight,
              child: Row(
                crossAxisAlignment: CrossAxisAlignment.start,
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  Container(
                    width: titleWidth,
                    padding: const EdgeInsets.symmetric(horizontal: 5),
                    child: ListView(
                      controller: ScrollController(),
                      scrollDirection: Axis.horizontal,
                      children: [
                        sizedBoxW16,
                        Center(
                          child: SelectableText(
                            widget.title,
                            style: const TextStyle(
                              fontWeight: FontWeight.bold,
                              fontSize: 20,
                            ),
                          ),
                        ),
                        sizedBoxW32,
                      ],
                    ),
                  ),
                  Container(
                    padding: const EdgeInsets.symmetric(horizontal: 5),
                    width: userActionsWidget,
                    child: FadeEdges(
                      child: ListView(
                        controller: ScrollController(),
                        scrollDirection: Axis.horizontal,
                        children: [
                          sizedBoxW8,
                          ...(widget.tableActions ?? []),
                          sizedBoxW8,
                        ],
                      ),
                    ),
                  ),
                  SizedBox(
                    width: coreActionsWidth,
                    child: ListView(
                      controller: ScrollController(),
                      reverse: true,
                      scrollDirection: Axis.horizontal,
                      children: [
                        sizedBoxW8,
                        Tooltip(
                          message: "Clear all filters",
                          child: IconButton(
                            onPressed: () {
                              setState(() {
                                _filterdData = _data;
                              });
                            },
                            icon: ThemedIcons.clear(),
                          ),
                        ),
                        sizedBoxW8,
                        Tooltip(
                          message: "Export as csv",
                          child: IconButton(
                            onPressed: () {
                              _dataToCsv();
                            },
                            icon: ThemedIcons.sheet(),
                          ),
                        ),
                        sizedBoxW8,
                        Container(
                            constraints: BoxConstraints(
                                maxHeight: _topBarHeight, maxWidth: 170),
                            child: TableSearchBar(
                              changeSearch: (String search) {
                                _debouncer.run(() {
                                  setState(() {
                                    _search = search;
                                    _currentPage = 0;
                                  });
                                });
                              },
                            )),
                        sizedBoxW8,
                      ],
                    ),
                  )
                ],
              ),
            ),
            ThemedDivider(height: _seperator, width: widget.frameWidth),
            SizedBox(
              height: widget.frameHeight -
                  (_topBarHeight + _bottomBarHeight + _seperator),
              width: widget.frameWidth,
              child: TableContent(
                filters: _filters,
                onFilterChanged: (Map<String, List<FilterField>> filters) {
                  setState(() {
                    _filters = filters;
                    _filterdData = _filterResult(_search, _filters, _data);
                  });
                },
                selectedListCallback: widget.selectedListCallback,
                data: _data,
                columns: widget.columns,
                showCopyRowButton: widget.showCopyRowButton,
                currentPage: _currentPage,
                handleSelectedRow: widget.handleSelectedRow,
                pageLength: _pageLength,
              ),
            ),
            SizedBox(
              height: _bottomBarHeight,
              width: widget.frameWidth,
              child: Row(
                crossAxisAlignment: CrossAxisAlignment.end,
                mainAxisAlignment: MainAxisAlignment.end,
                children: [
                  Stack(
                    children: [
                      const SizedBox(
                        height: 25,
                        width: 120,
                      ),
                      Positioned(
                        top: 0,
                        left: 0,
                        child: Row(
                          children: const [
                            SelectableText("Rows per page: "),
                            SizedBox(width: 7),
                          ],
                        ),
                      ),
                    ],
                  ),
                  SizedBox(
                    height: 30,
                    width: 100,
                    child: DropdownSearch<dynamic>(
                      mode: Mode.MENU,
                      showSearchBox: true,
                      items: widget.rowsPerPageOptions,
                      onChanged: (value) {
                        setState(() {
                          _pageLength = value;
                          _currentPage = 0;
                        });
                      },
                      selectedItem: _pageLength,
                    ),
                  ),
                  Stack(
                    children: [
                      const SizedBox(
                        height: 25,
                        width: 220,
                      ),
                      Positioned(
                        top: 0,
                        left: 10,
                        child: Row(
                          children: [
                            sizedBoxW16,
                            SelectableText(
                              "${((_currentPage * _pageLength) + 1).toString()} - ${_pageLength > itemsLength ? itemsLength.toString() : ((_currentPage + 1) * _pageLength).toString()} of  ${itemsLength.toString()} ",
                            ),
                            sizedBoxW16,
                            SelectableText(
                              "Page: ${(_currentPage + 1).toString()} ",
                            ),
                            sizedBoxW16,
                          ],
                        ),
                      ),
                    ],
                  ),
                  Stack(
                    children: [
                      const SizedBox(
                        height: 30,
                        width: 70,
                      ),
                      Positioned(
                          top: 0,
                          left: 0,
                          child: Row(children: [
                            InkWell(
                              onTap: () {
                                if (_currentPage > 0) {
                                  setState(() {
                                    _currentPage = _currentPage - 1;
                                  });
                                }
                              },
                              child: ThemedIcons.arrowBack(
                                iconStyle: IconStyle(
                                  color: _currentPage > 0
                                      ? Colors.black
                                      : Colors.grey,
                                ),
                              ),
                            ),
                            InkWell(
                              onTap: () {
                                if (_currentPage <
                                    (dataLength ~/ _pageLength)) {
                                  setState(() {
                                    _currentPage = _currentPage + 1;
                                  });
                                }
                              },
                              child: ThemedIcons.arrowForwared(
                                iconStyle: IconStyle(
                                  color: _currentPage < maxPages
                                      ? Colors.black
                                      : Colors.grey,
                                ),
                              ),
                            ),
                            sizedBoxW16
                          ])),
                    ],
                  ),
                ],
              ),
            ),
          ]),
    );
  }
}

class TableSearchBar extends StatelessWidget {
  final Function(String val) changeSearch;
  const TableSearchBar({Key? key, required this.changeSearch})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return TextField(
      onChanged: changeSearch,
      decoration: InputDecoration(
        suffixIcon: ThemedIcons.search(),
      ),
    );
  }
}

==> 0.0.27.tar.gz: lib/src/widgets/tables/table_frame.dart <==
import 'dart:convert';
import 'dart:typed_data';
import "package:collection/collection.dart";
import 'package:verttice_ui/src/const/box_sizes.dart';
import 'package:verttice_ui/src/const/icons.dart';
import 'package:verttice_ui/src/utils/download.dart';
import 'package:verttice_ui/src/widgets/fade_edges.dart';
import 'package:verttice_ui/src/widgets/form_factory/form_factory.dart';
import 'package:verttice_ui/src/widgets/tables/table_content.dart';
import 'package:verttice_ui/src/widgets/tables/table_utils.dart';
import "package:csv/csv.dart";
import "package:dropdown_search/dropdown_search.dart";
import "package:flutter/material.dart";
import "package:flutter/scheduler.dart";
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/src/widgets/themed_divider.dart';

class TableFrame extends StatefulWidget {
  final bool showCopyRowButton;
  final void Function(List<Map<String, dynamic>>)? selectedListCallback;
  final List<FormItem> columns;
  final List<Map<String, dynamic>> data;
  final double frameWidth;
  final double frameHeight;
  final List<int> rowsPerPageOptions;
  final void Function(
    Map<String, dynamic>? rowData,
    int rowIndex,
  ) handleSelectedRow;
  final List<Widget>? tableActions;
  final double? tableActionPrecentageWidth;
  final String title;
  final String uniqueIdField;

  const TableFrame({
    Key? key,
    required this.showCopyRowButton,
    required this.selectedListCallback,
    required this.columns,
    required this.data,
    required this.frameWidth,
    required this.frameHeight,
    required this.rowsPerPageOptions,
    required this.handleSelectedRow,
    required this.title,
    required this.uniqueIdField,
    this.tableActions,
    this.tableActionPrecentageWidth,
  }) : super(key: key);

  @override
  _TableFrameState createState() => _TableFrameState();
}

class _TableFrameState extends State<TableFrame> {
  final _debouncer = Debouncer(milliseconds: 500);
  final double _topBarHeight = 40;
  final double _bottomBarHeight = 40;
  final double _seperator = 2;
  late int _pageLength;
  int _currentPage = 0;
  late List<Map<String, dynamic>> _data;
  late List<Map<String, dynamic>> _filterdData;
  String _search = "";
  Map<String, List<FilterField>> _filters = {};

  @override
  void initState() {
    _pageLength = widget.rowsPerPageOptions[0];
    _data = widget.data;
    _filterdData = _data;
    super.initState();
  }

  @override
  void didUpdateWidget(covariant TableFrame oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const DeepCollectionEquality().equals(_data, widget.data)) {
      _data = widget.data;
      _filterdData = _filterResult(_search, _filters, _data);
    }
  }

  @override
  void setState(VoidCallback fn) {
    if (mounted) {
      SchedulerBinding.instance?.addPostFrameCallback((_) {
        super.setState(fn);
      });
    }
  }

  List<Map<String, dynamic>> _filterResult(
    String search,
    Map<String, List<FilterField>> filters,
    List<Map<String, dynamic>> data,
  ) {
    final result = _searchResult(search, _data).where((rowData) {
      bool include = true;
      for (var filterList in _filters.entries) {
        for (var filter in filterList.value) {
          if (!applayTableFilter(
            filter: filter,
            cellData: rowData[filter.field],
          )) {
            include = false;
          }
        }
      }
      return include;
    }).toList();

    return result;
  }

  List<Map<String, dynamic>> _searchResult(
    String search,
    List<Map<String, dynamic>> data,
  ) {
    return search == ""
        ? data
        : data
            .where(
              (rowData) => json.encode(rowData).toLowerCase().contains(
                    search.toLowerCase(),
                  ),
            )
            .toList();
  }

  void _dataToCsv() async {
    List<String> header = [];

    for (var element in widget.columns) {
      if (element.field.displayName != "") {
        header.add(element.field.displayName);
      }
    }

    final List<List<String>> data = [];

    data.add(header);

    _filterdData.asMap().forEach((rowIndex, rowData) {
      final List<String> currenRow = [];

      widget.columns.asMap().forEach((index, column) {
        currenRow.add(visualDataProcessor(column, rowData).value);
      });

      data.add(currenRow);
    });

    String csv = const ListToCsvConverter().convert(data);
    await Download.memory(
      data: Uint8List.fromList(csv.codeUnits),
      fileType: VFileType.csv,
    );
  }

  @override
  Widget build(BuildContext context) {
    final dataLength = _filterdData.length;
    final maxPages = (dataLength ~/ _pageLength);
    final itemsLength = dataLength;
    double titleWidth = widget.frameWidth / 3;
    double userActionsWidget = widget.frameWidth / 3;
    double coreActionsWidth = widget.frameWidth / 3;
    if (widget.tableActionPrecentageWidth != null &&
        widget.tableActionPrecentageWidth! <= 0.7) {
      userActionsWidget =
          widget.frameWidth * widget.tableActionPrecentageWidth!;

      titleWidth =
          widget.frameWidth * ((1.0 - widget.tableActionPrecentageWidth!) / 2);
      coreActionsWidth =
          widget.frameWidth * ((1.0 - widget.tableActionPrecentageWidth!) / 2);
    }

    return SizedBox(
      width: widget.frameWidth,
      height: widget.frameHeight,
      child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisAlignment: MainAxisAlignment.start,
          children: [
            SizedBox(
              width: widget.frameWidth,
              height: _topBarHeight,
              child: Row(
                crossAxisAlignment: CrossAxisAlignment.start,
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  Container(
                    width: titleWidth,
                    padding: const EdgeInsets.symmetric(horizontal: 5),
                    child: ListView(
                      controller: ScrollController(),
                      scrollDirection: Axis.horizontal,
                      children: [
                        sizedBoxW16,
                        Center(
                          child: SelectableText(
                            widget.title,
                            style: const TextStyle(
                              fontWeight: FontWeight.bold,
                              fontSize: 20,
                            ),
                          ),
                        ),
                        sizedBoxW32,
                      ],
                    ),
                  ),
                  Container(
                    padding: const EdgeInsets.symmetric(horizontal: 5),
                    width: userActionsWidget,
                    child: FadeEdges(
                      child: ListView(
                        controller: ScrollController(),
                        scrollDirection: Axis.horizontal,
                        children: [
                          sizedBoxW8,
                          ...(widget.tableActions ?? []),
                          sizedBoxW8,
                        ],
                      ),
                    ),
                  ),
                  SizedBox(
                    width: coreActionsWidth,
                    child: ListView(
                      controller: ScrollController(),
                      reverse: true,
                      scrollDirection: Axis.horizontal,
                      children: [
                        sizedBoxW8,
                        Tooltip(
                          message: "Clear all filters",
                          child: IconButton(
                            onPressed: () {
                              setState(() {
                                _filterdData = _data;
                                _filters = {};
                                _search = '';
                              });
                            },
                            icon: ThemedIcons.clear(),
                          ),
                        ),
                        sizedBoxW8,
                        Tooltip(
                          message: "Export as csv",
                          child: IconButton(
                            onPressed: () {
                              _dataToCsv();
                            },
                            icon: ThemedIcons.sheet(),
                          ),
                        ),
                        sizedBoxW8,
                        Container(
                            constraints: BoxConstraints(
                                maxHeight: _topBarHeight, maxWidth: 170),
                            child: TableSearchBar(
                              changeSearch: (String search) {
                                _debouncer.run(() {
                                  setState(() {
                                    _search = search;
                                    _currentPage = 0;
                                  });
                                });
                              },
                            )),
                        sizedBoxW8,
                      ],
                    ),
                  )
                ],
              ),
            ),
            ThemedDivider(height: _seperator, width: widget.frameWidth),
            SizedBox(
              height: widget.frameHeight -
                  (_topBarHeight + _bottomBarHeight + _seperator),
              width: widget.frameWidth,
              child: TableContent(
                filters: _filters,
                onFilterChanged: (Map<String, List<FilterField>> filters) {
                  setState(() {
                    _filters = filters;
                    _filterdData = _filterResult(_search, _filters, _data);
                    _currentPage = 0;
                  });
                },
                uniqueIdField: widget.uniqueIdField,
                selectedListCallback: widget.selectedListCallback,
                data: _data,
                columns: widget.columns,
                showCopyRowButton: widget.showCopyRowButton,
                currentPage: _currentPage,
                handleSelectedRow: widget.handleSelectedRow,
                pageLength: _pageLength,
              ),
            ),
            SizedBox(
              height: _bottomBarHeight,
              width: widget.frameWidth,
              child: Row(
                crossAxisAlignment: CrossAxisAlignment.end,
                mainAxisAlignment: MainAxisAlignment.end,
                children: [
                  Stack(
                    children: [
                      const SizedBox(
                        height: 25,
                        width: 120,
                      ),
                      Positioned(
                        top: 0,
                        left: 0,
                        child: Row(
                          children: const [
                            SelectableText("Rows per page: "),
                            SizedBox(width: 7),
                          ],
                        ),
                      ),
                    ],
                  ),
                  SizedBox(
                    height: 30,
                    width: 100,
                    child: DropdownSearch<dynamic>(
                      mode: Mode.MENU,
                      showSearchBox: true,
                      items: widget.rowsPerPageOptions,
                      onChanged: (value) {
                        setState(() {
                          _pageLength = value;
                          _currentPage = 0;
                        });
                      },
                      selectedItem: _pageLength,
                    ),
                  ),
                  Stack(
                    children: [
                      const SizedBox(
                        height: 25,
                        width: 220,
                      ),
                      Positioned(
                        top: 0,
                        left: 10,
                        child: Row(
                          children: [
                            sizedBoxW16,
                            SelectableText(
                              "${((_currentPage * _pageLength) + 1).toString()} - ${_pageLength > itemsLength ? itemsLength.toString() : ((_currentPage + 1) * _pageLength).toString()} of  ${itemsLength.toString()} ",
                            ),
                            sizedBoxW16,
                            SelectableText(
                              "Page: ${(_currentPage + 1).toString()} ",
                            ),
                            sizedBoxW16,
                          ],
                        ),
                      ),
                    ],
                  ),
                  Stack(
                    children: [
                      const SizedBox(
                        height: 30,
                        width: 70,
                      ),
                      Positioned(
                          top: 0,
                          left: 0,
                          child: Row(children: [
                            InkWell(
                              onTap: () {
                                if (_currentPage > 0) {
                                  setState(() {
                                    _currentPage = _currentPage - 1;
                                  });
                                }
                              },
                              child: ThemedIcons.arrowBack(
                                iconStyle: IconStyle(
                                  color: _currentPage > 0
                                      ? Colors.black
                                      : Colors.grey,
                                ),
                              ),
                            ),
                            InkWell(
                              onTap: () {
                                if (_currentPage <
                                    (dataLength ~/ _pageLength)) {
                                  setState(() {
                                    _currentPage = _currentPage + 1;
                                  });
                                }
                              },
                              child: ThemedIcons.arrowForwared(
                                iconStyle: IconStyle(
                                  color: _currentPage < maxPages
                                      ? Colors.black
                                      : Colors.grey,
                                ),
                              ),
                            ),
                            sizedBoxW16
                          ])),
                    ],
                  ),
                ],
              ),
            ),
          ]),
    );
  }
}

class TableSearchBar extends StatelessWidget {
  final Function(String val) changeSearch;
  const TableSearchBar({Key? key, required this.changeSearch})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return TextField(
      onChanged: changeSearch,
      decoration: InputDecoration(
        suffixIcon: ThemedIcons.search(),
      ),
    );
  }
}
